#include <QtGui>

#include "ScoreButton.h"
#include "EditScoreButtonDialog.h"

QString	EditScoreButtonDialog::lastIconPath = "";
EditScoreButtonDialog::EditScoreButtonDialog(QWidget *parent)
	: QDialog(parent)
{
	teamIconLabel			= new QLabel(tr("Team &Icon"),this);
	teamIconCheckbox		= new QCheckBox(this);
	teamIconCheckbox->setCheckState(Qt::Checked);
	connect(teamIconCheckbox, SIGNAL(stateChanged(int)), this, SLOT(showTeamIcon(int)));
	
	teamIconChooseButton	= new QPushButton(tr("Choose"),this);
	connect(teamIconChooseButton, SIGNAL(clicked()), this, SLOT(setTeamIcon()));
	teamIconLabel->setBuddy(teamIconChooseButton);
	
	iconWidthLabel			= new QLabel(tr("Icon &Width"),this);;
	iconWidthLineEdit		= new QLineEdit(this);
	iconWidthLabel->setBuddy(iconWidthLineEdit);
	QRegExp	sizeRegExp("[1-9][0-9]{1,3}");
	iconWidthLineEdit->setValidator(new QRegExpValidator(sizeRegExp,this));
	connect(iconWidthLineEdit, SIGNAL(textChanged (const QString&)), this, SLOT(setIconSize()));
	
	iconHeightLabel			= new QLabel(tr("Icon &Height"),this);;
	iconHeightLineEdit		= new QLineEdit(this);
	iconHeightLabel->setBuddy(iconHeightLineEdit);

	iconHeightLineEdit->setValidator(new QRegExpValidator(sizeRegExp,this));
	connect(iconHeightLineEdit, SIGNAL(textChanged (const QString&)), this, SLOT(setIconSize()));
	
	
	
	// ------------------------
	teamNameLabel			= new QLabel(tr("Team &Name"), this);
	teamNameLineEdit		= new QLineEdit(tr("Team Name"), this);
	connect(teamNameLineEdit, SIGNAL(textChanged (const QString&)), this, SLOT(setTeamName()));
	teamNameLabel->setBuddy(teamNameLineEdit);	
	
	//-----------------------
	teamScoreLabel			= new QLabel(tr("Team &Score"), this);
	teamScoreLineEdit		= new QLineEdit(tr("0"), this);
	QRegExp	scoreRegExp("[0-9]{1,5}");
	teamScoreLineEdit->setValidator(new QRegExpValidator(scoreRegExp,this));
	connect(teamScoreLineEdit, SIGNAL(textChanged (const QString&)), this, SLOT(setTeamScore()));
	teamScoreLabel->setBuddy(teamScoreLineEdit);
	
	//----------------------
	smileyLabel			= new QLabel(tr("Number of Smile&y"), this);
	smileyLineEdit		= new QLineEdit(tr("0"), this);
	QRegExp	smileyRegExp("[0-9]");
	smileyLineEdit->setValidator(new QRegExpValidator(smileyRegExp,this));
	connect(smileyLineEdit, SIGNAL(textChanged (const QString&)), this, SLOT(setSmiley()));
	smileyLabel->setBuddy(smileyLineEdit);	
	//----------------------
	setNameFontButton			= new QPushButton(tr("&Font for Name"),this);
	connect(setNameFontButton, SIGNAL(clicked()), this, SLOT(setTeamNameFont()));
	setNameColorButton			= new QPushButton(tr("&Color for Name"),this);
	connect(setNameColorButton, SIGNAL(clicked()), this, SLOT(setTeamNameColor()));
	
	//-----------------------
	setScoreFontButton			= new QPushButton(tr("Fon&t for Score"),this);
	connect(setScoreFontButton, SIGNAL(clicked()), this, SLOT(setTeamScoreFont()));
	setScoreColorButton			= new QPushButton(tr("Colo&r for Score"),this);
	connect(setScoreColorButton, SIGNAL(clicked()), this, SLOT(setTeamScoreColor()));
		
	//------------------
	previewLabel			= new QLabel(tr("Preview"),this);
	previewScoreButton		= new ScoreButton(this);
	previewScoreButton->setPreviewMode();
	
	okButton				= new QPushButton(tr("&Accept"),this);
	okButton->setDefault(true);
	cancelButton			= new QPushButton(tr("&Reject"),this);
	connect(okButton, SIGNAL(clicked()), this, SLOT(accept()));
	connect(cancelButton, SIGNAL(clicked()), this, SLOT(reject()));
	updateAll();
}
void EditScoreButtonDialog::setupLayout()
{
	if(layout()!=0)
	{
		delete layout();
	}
	
	QHBoxLayout *iconLayout = new QHBoxLayout;
	iconLayout->addWidget(teamIconLabel);
	iconLayout->addWidget(teamIconCheckbox);
	iconLayout->addWidget(teamIconChooseButton);
	
	QHBoxLayout *iconWidthLayout = new QHBoxLayout;
	iconWidthLayout->addWidget(iconWidthLabel);
	iconWidthLayout->addWidget(iconWidthLineEdit);

	QHBoxLayout *iconHeightLayout = new QHBoxLayout;
	iconHeightLayout->addWidget(iconHeightLabel);
	iconHeightLayout->addWidget(iconHeightLineEdit);
	
	QHBoxLayout *nameLayout = new QHBoxLayout;
	nameLayout->addWidget(teamNameLabel);
	nameLayout->addWidget(teamNameLineEdit);

	QHBoxLayout *teamScoreLayout = new QHBoxLayout;
	teamScoreLayout->addWidget(teamScoreLabel);
	teamScoreLayout->addWidget(teamScoreLineEdit);

	QHBoxLayout *smileyLayout = new QHBoxLayout;
	smileyLayout->addWidget(smileyLabel);
	smileyLayout->addWidget(smileyLineEdit);

	QHBoxLayout *nameFontLayout = new QHBoxLayout;
	nameFontLayout->addWidget(setNameFontButton);
	nameFontLayout->addWidget(setNameColorButton);
	
	QHBoxLayout *scoreFontLayout = new QHBoxLayout;
	scoreFontLayout->addWidget(setScoreFontButton);
	scoreFontLayout->addWidget(setScoreColorButton);
	
	QHBoxLayout *okLayout = new QHBoxLayout;
	okLayout->addStretch(1);
	okLayout->addWidget(okButton);
	okLayout->addWidget(cancelButton);	
	
	QVBoxLayout *mainLayout = new QVBoxLayout;
	mainLayout->addLayout(iconLayout);
	mainLayout->addLayout(iconWidthLayout);
	mainLayout->addLayout(iconHeightLayout);
	mainLayout->addLayout(nameLayout);
	mainLayout->addLayout(teamScoreLayout);
	mainLayout->addLayout(smileyLayout);
	mainLayout->addLayout(nameFontLayout);
	mainLayout->addLayout(scoreFontLayout);
	mainLayout->addWidget(previewLabel);
	mainLayout->addWidget(previewScoreButton,1);
	mainLayout->addLayout(okLayout);
	
	setLayout(mainLayout);
}
void EditScoreButtonDialog::updateAll()
{
	teamNameLineEdit->setText(previewScoreButton->getTeamName());
	teamScoreLineEdit->setText(QString("%1").arg(previewScoreButton->getTeamScore()));
	smileyLineEdit->setText(QString("%1").arg(previewScoreButton->getNumberOfSmiley()));
	if(previewScoreButton->teamIconShown())
	{
		teamIconCheckbox->setCheckState(Qt::Checked);
	}
	else
	{
		teamIconCheckbox->setCheckState(Qt::Unchecked);
	}
	QSize	iconSize = previewScoreButton->getIconSize();
	iconWidthLineEdit->setText(QString("%1").arg(iconSize.width()));
	iconHeightLineEdit->setText(QString("%1").arg(iconSize.height()));
	
	setupLayout();
}
void EditScoreButtonDialog::showTeamIcon(int show)
{
	if(show==Qt::Checked)
	{
		previewScoreButton->showTeamIcon(true);
	}
	else
	{
		previewScoreButton->showTeamIcon(false);
	}
	//updateAll();
}
void EditScoreButtonDialog::setSmiley()
{
	if(layout()!=0)
	{
		delete layout();
	}
	
	if(!smileyLineEdit->text().isEmpty())
	{	//
		previewScoreButton->setNumberOfSmiley(smileyLineEdit->text().toInt());
	}
	updateAll();
}
void EditScoreButtonDialog::setIconSize()
{
	if((!iconWidthLineEdit->text().isEmpty())&&(!iconHeightLineEdit->text().isEmpty()))
	{
		previewScoreButton->setIconSize(QSize(iconWidthLineEdit->text().toInt(),iconHeightLineEdit->text().toInt()));
	}
}

void EditScoreButtonDialog::setTeamScore()
{
	if(!teamScoreLineEdit->text().isEmpty())
	{
		previewScoreButton->setTeamScore(teamScoreLineEdit->text().toInt());
	}
}
void EditScoreButtonDialog::setTeamName()
{
	previewScoreButton->setTeamName(teamNameLineEdit->text());
}
void EditScoreButtonDialog::setTeamIcon()
{
	//QMessageBox::information(this, tr("Score Board"), tr("set team icon"));
	if(EditScoreButtonDialog::lastIconPath.isEmpty())
	{
		EditScoreButtonDialog::lastIconPath = QDir::currentPath();
	}
	QString fileName = QFileDialog::getOpenFileName(this, tr("Load Picture File"), EditScoreButtonDialog::lastIconPath ,"Images (*.png *.xpm *.jpg *bmp)");
	
	if (!fileName.isEmpty())
	{
		QPixmap newIcon(fileName);
		if (newIcon.isNull())
		{
			QMessageBox::information(this, tr("Score Board"), tr("%1 is not a valid picture file").arg(fileName));
			return;
		}
		previewScoreButton->setTeamIcon(newIcon);
		EditScoreButtonDialog::lastIconPath = QFileInfo(fileName).absoluteDir ().absolutePath () ;
	}
}
void EditScoreButtonDialog::setTeamNameFont()
{
	previewScoreButton->nameLabel->setFont(QFontDialog::getFont(0, previewScoreButton->nameLabel->font()));
}
void EditScoreButtonDialog::setTeamNameColor()
{
	QPalette pal = previewScoreButton->nameLabel->palette();
	QColor color = QColorDialog::getColor ( pal.color(QPalette::Foreground) );
	if(color.isValid())
	{
		pal.setColor(QPalette::Foreground, color );
	}

	previewScoreButton->nameLabel->setPalette(pal);
}

void EditScoreButtonDialog::setTeamScoreFont()
{
	previewScoreButton->scoreLabel->setFont(QFontDialog::getFont(0, previewScoreButton->scoreLabel->font()));
}
void EditScoreButtonDialog::setTeamScoreColor()
{
	QPalette pal = previewScoreButton->scoreLabel->palette();
	QColor color = QColorDialog::getColor ( pal.color(QPalette::Foreground) );
	if(color.isValid())
	{
		pal.setColor(QPalette::Foreground, color );
	}
	previewScoreButton->scoreLabel->setPalette(pal);
}
