#include <QtGui>
#include <QVBoxLayout>

#include "ScoreBoard.h"
#include "ScoreButton.h"

ScoreBoard::ScoreBoard()
{
    createActions();
    createMenus();
    setWindowTitle(tr("Score Board"));
	setWindowIcon(QPixmap(":/images/main_icon.png"));
    resize(300, 600);

	setCentralWidget(new QWidget);
	fileName = "";
	newFile();
}
void ScoreBoard::populateCentralWidget()
{
	if(centralWidget()->layout()!=0) // if the central widget already have a layout
	{
		delete centralWidget()->layout(); // delete the layout
	}
	
	QVBoxLayout *layout = new QVBoxLayout;
	centralWidget()->setLayout(layout);

	for (int i = 0; i < groupList.size(); ++i)
	{
		centralWidget()->layout()->addWidget(groupList[i]); // add the list of ScoreButton to the layout
		groupList[i]->show();
	}
}

void ScoreBoard::newFile()
{
	// Clear all the ScoreButton class
	while(!groupList.isEmpty())
	{
		delete groupList.takeFirst();
		//centralWidget()->layout()->removeWidget(temp);
	}
	
	ScoreButton	*tempButton = new ScoreButton(centralWidget());
	tempButton->setAttribute(Qt::WA_DeleteOnClose);
	tempButton->setNumberOfSmiley(3);
	tempButton->setButtonID(0);
	groupList.append(tempButton);
	
	connect(tempButton, SIGNAL(addButton(int,bool)), this, SLOT(addButton(int,bool)));
	connect(tempButton, SIGNAL(deleteButton(int)), this, SLOT(deleteButton(int)));
	
	connect(tempButton, SIGNAL(moveUpButton(int)), this, SLOT(moveUpButton(int)));
	connect(tempButton, SIGNAL(moveDownButton(int)), this, SLOT(moveDownButton(int)));
	
	populateCentralWidget();
}

void ScoreBoard::open()
{
	QString lastPath;
	if(!fileName.isEmpty())
	{
		lastPath = QFileInfo(fileName).absoluteDir ().absolutePath () ;
	}
	else
	{
		lastPath = QDir::currentPath();
	}
	
    QString loadName = QFileDialog::getOpenFileName(this, tr("Open..."), lastPath, "Score Board File (*.scb)");
    if (!loadName.isEmpty())
	{
        
        if (loadFile(loadName)) 
		{
            fileName = loadName;
            return;
        }
        else
		{
			QMessageBox::information(this, tr("Score Board"), tr("Cannot load %1.").arg(loadName));
		}
    }

}
bool ScoreBoard::saveToFile(QString name)
{
	QFile	file(name);
	if(!file.open(QIODevice::WriteOnly))
	{

		return false;
	}
	
	QDataStream	out(&file);
	out.setVersion(7);
	
	out<< (quint32)MagicNumber;
	//	number of buttons
	int buttonCount = groupList.size();
	out << buttonCount;
	for(int i=0;i<groupList.size();i++)
	{
		out<<groupList[i]->getTeamName();
		out<<groupList[i]->getTeamScore();
		out<<groupList[i]->getNumberOfSmiley();
		out<<*(groupList[i]->teamIcon);
		out<<groupList[i]->teamIconShown();
		out<<groupList[i]->getIconSize();
		out<<groupList[i]->nameLabel->palette();
		out<<groupList[i]->nameLabel->font();
		out<<groupList[i]->scoreLabel->palette();
		out<<groupList[i]->scoreLabel->font();
	}

	return true;
}
bool ScoreBoard::loadFile(QString name)
{
	QFile	file(name);
	if(!file.open(QIODevice::ReadOnly))
	{
		return false;
	}
	
	QDataStream	in(&file);
	in.setVersion(7);
	
	quint32 magic;
	in>>magic;
	if(magic!=MagicNumber)
	{
		return false;
	}
	//	number of buttons
	int buttonCount;
	in>>buttonCount;
	
	// Clear all the ScoreButton
	while(!groupList.isEmpty())
	{
		delete groupList.takeFirst();
	}
	
	for(int i=0;i<buttonCount;i++)
	{
		ScoreButton	*tempButton = new ScoreButton(centralWidget());
		tempButton->setAttribute(Qt::WA_DeleteOnClose);
		
		QString teamName;
		in>>teamName;
		tempButton->setTeamName(teamName);
		
		int	teamScore;
		in>>teamScore;
		tempButton->setTeamScore(teamScore);
		
		int	numberOfSmiley;
		in>>numberOfSmiley;
		tempButton->setNumberOfSmiley(numberOfSmiley);
		
		QPixmap	teamIcon;
		in>>teamIcon;
		tempButton->setTeamIcon(teamIcon);
		
		bool	showTeamIcon;
		in>>showTeamIcon;
		tempButton->showTeamIcon(showTeamIcon);
		
		QSize	iconSize;
		in>>iconSize;
		tempButton->setIconSize(iconSize);
		
		QPalette	namePal;
		in>>namePal;
		tempButton->nameLabel->setPalette(namePal);
		
		QFont		nameFont;
		in>>nameFont;
		tempButton->nameLabel->setFont(nameFont);
		
		QPalette	scorePal;
		in>>scorePal;
		tempButton->scoreLabel->setPalette(scorePal);
		
		QFont		scoreFont;
		in>>scoreFont;
		tempButton->scoreLabel->setFont(scoreFont);
		
		tempButton->show();
	
		groupList.append(tempButton);
		connect(tempButton, SIGNAL(addButton(int,bool)), this, SLOT(addButton(int,bool)));
		connect(tempButton, SIGNAL(deleteButton(int)), this, SLOT(deleteButton(int)));
	
		connect(tempButton, SIGNAL(moveUpButton(int)), this, SLOT(moveUpButton(int)));
		connect(tempButton, SIGNAL(moveDownButton(int)), this, SLOT(moveDownButton(int)));

	}
	
	populateCentralWidget();
	return true;
}
void ScoreBoard::save()
{
	if(fileName.isEmpty())
	{
		saveAs();
		return;
	}
	else
	{
		if(saveToFile(fileName))
		{
			// save succeeded;
		}
		else
		{
			QMessageBox::warning(0,tr("Score Board"), tr("Cannot save to file: %1").arg(fileName));
			fileName = ""; // save failed
		}
	}
}
void ScoreBoard::saveAs()
{
	QString lastPath;
	if(!fileName.isEmpty())
	{
		lastPath = QFileInfo(fileName).absoluteDir ().absolutePath () ;
	}
	else
	{
		lastPath = QDir::currentPath();
	}
	
	QString saveFileName = QFileDialog::getSaveFileName(this, tr("Save As..."), lastPath ,"Score Board File (*.scb)");
	
	if (!saveFileName.isEmpty())
	{
		QFileInfo fi(saveFileName);
		QString ext = fi.completeSuffix();
		if(ext!="scb")
		{
			saveFileName+=".scb";
		}
		
		if (saveToFile(saveFileName))
		{
			//QMessageBox::information(this, tr("Score Board"), tr("%1 is not a valid picture file").arg(fileName));
			fileName = saveFileName;
			return;
		}
		else
		{
			QMessageBox::warning(0,tr("Score Board"), tr("Cannot save to file: %1").arg(saveFileName));
		}
	}
	else
	{
		//QMessageBox::warning(0,tr("Score Board"), tr("Cannot save to file: %1").arg(fileName));
		return;
	}
}
void ScoreBoard::toggleTopmost()
{
	Qt::WindowFlags flags = 0;
	flags = this->windowFlags();
	
	if(toggleTopmostAct->isChecked())
	{
		flags |= Qt::WindowStaysOnTopHint;
	}
	else
	{
		flags &= ~(Qt::WindowStaysOnTopHint);
	}
	this->setWindowFlags(flags);
	this->show();
}
void ScoreBoard::createActions()
{
    newFileAct = new QAction(QIcon(":/images/file_new.png"), tr("&New..."), this);
    newFileAct->setShortcut(tr("Ctrl+N"));
    connect(newFileAct, SIGNAL(triggered()), this, SLOT(newFile()));

    openAct = new QAction(QIcon(":/images/file_open.png"), tr("&Open..."), this);
    openAct->setShortcut(tr("Ctrl+O"));
    connect(openAct, SIGNAL(triggered()), this, SLOT(open()));

    saveAct = new QAction(QIcon(":/images/file_save.png"), tr("&Save..."), this);
    saveAct->setShortcut(tr("Ctrl+S"));
    connect(saveAct, SIGNAL(triggered()), this, SLOT(save()));

    saveAsAct = new QAction(QIcon(":/images/file_saveas.png"), tr("Save &As..."), this);
    saveAsAct->setShortcut(tr("Ctrl+Alt+S"));
    connect(saveAsAct, SIGNAL(triggered()), this, SLOT(saveAs()));

    exitAct = new QAction(QIcon(":/images/file_exit.png"), tr("E&xit"), this);
    exitAct->setShortcut(tr("Ctrl+Q"));
    connect(exitAct, SIGNAL(triggered()), this, SLOT(close()));

	toggleTopmostAct = new QAction(QIcon(":/images/view_topmost.png"), tr("&Topmost..."), this);
	toggleTopmostAct->setShortcut(tr("Ctrl+T"));
	toggleTopmostAct->setCheckable(true);
	toggleTopmostAct->setChecked(false);
	connect(toggleTopmostAct, SIGNAL(triggered()), this, SLOT(toggleTopmost()));
	
	rankButtonsAct = new QAction(QIcon(":/images/rank.png"), tr("&Rank..."), this);
	rankButtonsAct->setShortcut(tr("Ctrl+R"));
	connect(rankButtonsAct, SIGNAL(triggered()), this, SLOT(rankButtons()));
	
    aboutAct = new QAction(QIcon(":/images/main_icon.png"), tr("&About"), this);
    connect(aboutAct, SIGNAL(triggered()), this, SLOT(about()));
	
}

void ScoreBoard::createMenus()
{
    fileMenu = new QMenu(tr("&File"), this);
	fileMenu->addAction(newFileAct);
	fileMenu->addAction(openAct);
    fileMenu->addAction(saveAct);
    fileMenu->addAction(saveAsAct);
    fileMenu->addSeparator();
    fileMenu->addAction(exitAct);

	viewMenu = new QMenu(tr("&View"), this);
	viewMenu->addAction(toggleTopmostAct);
	viewMenu->addAction(rankButtonsAct);

    helpMenu = new QMenu(tr("&Help"), this);
    helpMenu->addAction(aboutAct);

    menuBar()->addMenu(fileMenu);
	menuBar()->addMenu(viewMenu);
    menuBar()->addMenu(helpMenu);
}

void ScoreBoard::updateActions()
{
//    zoomInAct->setEnabled(!fitToWindowAct->isChecked());
//    zoomOutAct->setEnabled(!fitToWindowAct->isChecked());
//    normalSizeAct->setEnabled(!fitToWindowAct->isChecked());
}
void ScoreBoard::addButton(int	buttonID, bool above)
{
	int	i=0;
	for(i=0;i<groupList.size();i++)
	{
		if(groupList[i]->getButtonID()==buttonID)
		{
			break;
		}
	}
	
	ScoreButton	*tempButton = new ScoreButton(centralWidget());
	tempButton->setAttribute(Qt::WA_DeleteOnClose);
	groupList[i]->copyTo(tempButton);
	
	if(above)
	{
		groupList.insert(i,tempButton);
	}
	else
	{
		groupList.insert(i+1,tempButton);
	}

	connect(tempButton, SIGNAL(addButton(int,bool)), this, SLOT(addButton(int,bool)));
	connect(tempButton, SIGNAL(deleteButton(int)), this, SLOT(deleteButton(int)));

	connect(tempButton, SIGNAL(moveUpButton(int)), this, SLOT(moveUpButton(int)));
	connect(tempButton, SIGNAL(moveDownButton(int)), this, SLOT(moveDownButton(int)));
	
	resetButtonID();
	populateCentralWidget();
}

void ScoreBoard::moveUpButton(int	buttonID)
{
	int	i=0;
	for(i=0;i<groupList.size();i++)
	{
		if(groupList[i]->getButtonID()==buttonID)
		{
			break;
		}
	}
	if(i>0)
	{
		ScoreButton* temp=groupList.takeAt(i);
		groupList.insert(i-1,temp);
		resetButtonID();
		populateCentralWidget();
	}
}
void ScoreBoard::moveDownButton(int	buttonID)
{
	int	i=0;
	for(i=0;i<groupList.size();i++)
	{
		if(groupList[i]->getButtonID()==buttonID)
		{
			break;
		}
	}
	if(i<(groupList.size()-1))
	{
		ScoreButton* temp=groupList.takeAt(i);
		groupList.insert(i+1,temp);
		resetButtonID();
		populateCentralWidget();
	}
}
void ScoreBoard::rankButtons()
{
	int	i=0;
	for(i=0;i<groupList.size();i++)
	{
		for(int j = i+1 ; j < groupList.size(); j++)
		{
			if(groupList[i]->getTeamScore() < groupList[j]->getTeamScore() )
			{
				//swap the two teams
				ScoreButton *temp = groupList[i];
				groupList[i] = groupList[j];
				groupList[j] = temp;
			}
			
		}
	}
	resetButtonID();
	populateCentralWidget();

}
void ScoreBoard::deleteButton(int	buttonID)
{
	
	if(groupList.size()==1)
	{
		QMessageBox::information(this, tr("Score Board"), tr("This is the last group. Score Board will keep at least one group.<br />Remove action cancelled."));
		return;
	}
	int	i=0;
	for(i=0;i<groupList.size();i++)
	{
		if(groupList[i]->getButtonID()==buttonID)
		{
			break;
		}
	}
	

	//delete	groupList[i]; // instead of delete, use close();
	groupList[i]->close();
	//delete	groupList[i];
	groupList.removeAt(i);
	
	resetButtonID();
	populateCentralWidget();
	
	//QMessageBox::information(this, tr("Score Board"), tr("%1 group").arg(groupList.size()));
}
void ScoreBoard::resetButtonID()
{
	for(int i=0;i<groupList.size();i++)
	{
		groupList[i]->setButtonID(i);
	}
}
void ScoreBoard::about()
{
    QMessageBox::about(this, tr("About Score Board"),
            tr("<p><H1>Score Board 0.2 Beta</H1></p>"
				"<p><b>Score Board</b> is wrtiten with QT 4.1.3 and publised under GPL.<br />This is my first QT program, there maybe memory leaks, obvious (or not-so-obvious) bugs. Please feel free to point them out for me.<br />My Email: dyzfromyt AT gmail DOT com</p>"
				"<p>Icons used in this application are taken from<br />"
				"<i>Crystal Clear</i> icon set<br />"
				"<a href=\"http://www.linuxcult.com/forum/crystal.php\">http://www.linuxcult.com/forum/crystal.php</a><br />"
				"and<br />"
				"<i>Emotion Icons</i><br />"
				"<a href=http://www.everaldo.com>http://www.everaldo.com</a>"
				"</p>"
				));
}
