#include <QtGui>
#include <QList>

#include "ScoreButton.h"
#include "SmileyButton.h"
#include "EditScoreButtonDialog.h"

ScoreButton::ScoreButton(QWidget *parent, Qt::WFlags f)
    : QWidget(parent, f)
{
	teamScore = 0;
	previewMode = false;
	
	teamName = QString (tr("New Team"));
	
	increaseScoreButton = new QToolButton(this);
	increaseScoreButton->setAutoRaise(true);
	increaseScoreButton->setIcon(QIcon(":/images/button_increase_score.png"));
	increaseScoreButton->setIconSize(QSize(22,22));
	connect(increaseScoreButton, SIGNAL(clicked()),
				this, SLOT(increaseScore()));
	
				
	decreaseScoreButton = new QToolButton(this);
	decreaseScoreButton->setAutoRaise(true);
	decreaseScoreButton->setIcon(QIcon(":/images/button_decrease_score.png"));
	decreaseScoreButton->setIconSize(QSize(22,22));
	connect(decreaseScoreButton, SIGNAL(clicked()),
				this, SLOT(decreaseScore()));
				
	
	scoreLabel = new QLabel(this);
				
	nameLabel = new QLabel(this);
	
	teamIcon = new QPixmap(":/images/button_default_icon.png");

	
	teamIconLabel = new QLabel(this);
	teamIconLabel->setFrameShape(QFrame::Box);
	teamIconLabel->setAlignment(Qt::AlignCenter);
	

	teamIconLabel->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
	teamIconLabel->setScaledContents(true);
	teamIconLabel->setFixedSize(QSize(TEAMICON_SIZE, TEAMICON_SIZE));
	showTeamIcon(true);
    //label->setBackgroundRole(QPalette::Base);
    //label->setMinimumSize(132, 132);
		
	
	connect(this, SIGNAL(scoreChanged(int)),
				this, SLOT(updateScore()));
				
	updateAll();
}

void ScoreButton::setPreviewMode()
{
	previewMode = true;
}
void ScoreButton::addGroup()
{
	//QMessageBox::about(this, tr("About Score Board"),tr("add group"));
	emit	addButton(buttonID, false); // fasle means add one button below the current button
}
void ScoreButton::editGroup()
{
	EditScoreButtonDialog ed(this);
	this->copyTo(ed.previewScoreButton);
	ed.updateAll();
	if(ed.exec()==QDialog::Accepted)
	{
		ed.previewScoreButton->copyTo(this);
	}
	//QMessageBox::about(this, tr("About Score Board"),tr("edit group"));
}
void ScoreButton::deleteGroup()
{
	//QMessageBox::about(this, tr("About Score Board"),tr("delete group"));
	emit	deleteButton(buttonID);
}
void ScoreButton::moveUpGroup()
{
	emit	moveUpButton(buttonID);
}

void ScoreButton::moveDownGroup()
{
	emit	moveDownButton(buttonID);
}
void ScoreButton::showTeamIcon(bool show)
{
	boolShowTeamIcon = show;
	if(show)
	{
		//teamIconLabel->setFixedSize(QSize(TEAMICON_SIZE, TEAMICON_SIZE));
		teamIconLabel->show();
	}
	else
	{
		//teamIconLabel->setFixedSize(QSize(0, 0));
		teamIconLabel->hide();
	}
}
bool ScoreButton::teamIconShown() const
{
	return	boolShowTeamIcon;
}
void ScoreButton::copyTo(ScoreButton* target) // just making them looks like, they are different objects, buttonID is not copied
{
	target->setNumberOfSmiley(getNumberOfSmiley());
	target->teamName = teamName;
	target->teamScore = teamScore;

	target->setTeamIcon(*teamIcon);
	target->showTeamIcon(teamIconShown());
	target->setIconSize(getIconSize());
	
	target->nameLabel->setPalette(this->nameLabel->palette());
	target->nameLabel->setFont(this->nameLabel->font());
	
	target->scoreLabel->setPalette(this->scoreLabel->palette());
	target->scoreLabel->setFont(this->scoreLabel->font());
}
void ScoreButton::setupLayout()
{
	if(layout()!=0)
	{
		delete layout();
	}
	QHBoxLayout *mainLayout = new QHBoxLayout;
	QVBoxLayout *pmLayout = new QVBoxLayout;  // Layout for the "plus score" "minus score" button
	QVBoxLayout *teamNameLayout = new QVBoxLayout;	// Layout for the team name, it include the smileys
	
	if(!smiley.isEmpty())
	{
		QHBoxLayout *smileyLayout = new QHBoxLayout; // Layout for the smileys
		for(int i=0;i<smiley.size();i++)
		{
			smileyLayout->addWidget(smiley[i]);
			smiley[i]->show();
		}
		teamNameLayout->addStretch(1);
		teamNameLayout->addWidget(nameLabel);
		teamNameLayout->addLayout(smileyLayout);
		teamNameLayout->addStretch(1);
	}
	else
	{
		teamNameLayout->addWidget(nameLabel);
	}
	pmLayout->addStretch(1);
	pmLayout->addWidget(increaseScoreButton);
	pmLayout->addWidget(decreaseScoreButton);
	pmLayout->addStretch(1);

	mainLayout->addWidget(teamIconLabel);
	mainLayout->addLayout(teamNameLayout,1);
	mainLayout->addWidget(scoreLabel);
	mainLayout->addLayout(pmLayout);
	

	setLayout(mainLayout);

	
}
void ScoreButton::setIconSize(const QSize& size)
{
	teamIconLabel->setFixedSize(size);
}
QSize ScoreButton::getIconSize() const
{
	return	teamIconLabel->size();
}
void ScoreButton::setTeamIcon(const QPixmap& newIcon)
{
	if(!newIcon.isNull())
	{
		delete	teamIcon;
		teamIcon = new QPixmap(newIcon);
	}
	updateAll();
}

void ScoreButton::setNumberOfSmiley(const int num)
{
	while(!smiley.isEmpty())
	{
		delete	smiley.takeFirst();
	}
	
	for(int i=0;i<num;i++)
	{
		SmileyButton* temp=0;
		temp = new SmileyButton(this);
		if(temp==0)
		{
			QMessageBox::information(this, tr("Score Board"), tr("fail to allocate SmileyButton"));
		}
		smiley.append(temp);
	}

	updateAll();
}
int	ScoreButton::getNumberOfSmiley()	const
{
	return	smiley.size();
}
void ScoreButton::contextMenuEvent(QContextMenuEvent * e)
{
	if(previewMode == true)
	{
		return;
	}
	QMenu menu;
	QAction* moveUpAct;
	moveUpAct = new QAction(QIcon(":/images/context_up_group.png"), tr("Move &Up"), this);
	connect(moveUpAct, SIGNAL(triggered()), this, SLOT(moveUpGroup()));

	QAction* moveDownAct;
	moveDownAct = new QAction(QIcon(":/images/context_down_group.png"), tr("Move Dow&n"), this);
	connect(moveDownAct, SIGNAL(triggered()), this, SLOT(moveDownGroup()));

	QAction* addGroupAct;
	addGroupAct = new QAction(QIcon(":/images/context_add_group.png"), tr("&Add New Group"), this);
	connect(addGroupAct, SIGNAL(triggered()), this, SLOT(addGroup()));
	
	QAction* editGroupAct;
	editGroupAct = new QAction(QIcon(":/images/context_edit_group.png"), tr("&Edit This Group"), this);
	connect(editGroupAct, SIGNAL(triggered()), this, SLOT(editGroup()));
	
	QAction* deleteGroupAct;
	deleteGroupAct = new QAction(QIcon(":/images/context_delete_group.png"), tr("&Delete This Group"), this);
	connect(deleteGroupAct, SIGNAL(triggered()), this, SLOT(deleteGroup()));
	
	menu.addAction(moveUpAct);
	menu.addAction(moveDownAct);
	menu.addSeparator();
	menu.addAction(addGroupAct);
	menu.addAction(editGroupAct);
	menu.addSeparator();
	menu.addAction(deleteGroupAct);
	
	menu.exec(mapToGlobal(e->pos()));
	
	delete	addGroupAct;
	delete	editGroupAct;
	delete	deleteGroupAct;
}

void ScoreButton::updateScore()
{
	scoreLabel->setText(QString(":%1").arg(teamScore));
}

void ScoreButton::updateAll()
{
	nameLabel->setText(teamName);
	teamIconLabel->setPixmap(*teamIcon);
	updateScore();
	setupLayout();
}

void ScoreButton::setTeamName(const QString &newTeamName)
{
	teamName = newTeamName;
	updateAll();
}
QString	ScoreButton::getTeamName() const
{
	return	teamName;
}

void ScoreButton::setTeamScore(const int score)
{
	if(teamScore == score)
	{
		teamScore = score;
	}
	else
	{
		teamScore = score;
		emit	scoreChanged(teamScore);
	}
}

void ScoreButton::increaseScore()
{
	teamScore ++;
	emit	scoreChanged(teamScore);
}

void ScoreButton::decreaseScore()
{
	teamScore --;
	emit	scoreChanged(teamScore);
}

int ScoreButton::getTeamScore()
{
	return	teamScore;
}
void ScoreButton::setButtonID(int newID)
{
	buttonID = newID;
}
int	ScoreButton::getButtonID() const
{
	return buttonID;
}

